import Vue from 'vue';
import VueI18n from 'vue-i18n';

import Cookie from '@/common/cookie';
import {deepFreeze, getUrlParameter} from '@/common/util';

// import elementZHTW from 'element-ui/lib/locale/lang/zh-TW';
import elementZHCH from 'element-ui/lib/locale/lang/zh-CN';
import elementEn from 'element-ui/lib/locale/lang/en';

// import zhHKLang from '@/locales/zh-HK';
import zhCNLang from '@/locales/zh-CN';
import enLang from '@/locales/en';

Vue.use(VueI18n);

const messages = {
  // 'zh-HK': {
  //   ...deepFreeze(elementZHTW),
  //   ...zhHKLang,
  // },
  'zh-CN': {
    ...deepFreeze(elementZHCH),
    ...zhCNLang,
  },
  en: {
    ...deepFreeze(elementEn),
    ...enLang,
  },
};

/**
 * 根据传入的 language code 确定受项目支持的语言
 * 如匹配不到，返回空字符串
 * @param {string} lang
 */
function getSupportLang(lang = '') {
  if (typeof lang !== 'string') {
    return '';
  }
  lang = lang.toLowerCase();
  const locales = [
    // {
    //   lang: 'zh-HK',
    //   matches: ['zh-HK', 'zh-TW', 'zh-MO', 'Hant'],
    // },
    {
      lang: 'zh-CN',
      matches: ['zh-CN', 'zh-SG', 'Hans'],
    },
    {
      lang: 'en',
      matches: ['en'],
    },
  ];

  let matched = locales.find(locale => {
    return locale.matches.some(item => {
      return lang.includes(item.toLowerCase());
    });
  });

  return (matched && matched.lang) || '';
}

/**
 * 决定当前语言
 *  1. 读取语言标识：优先读取 url `lang`，其次读取 cookie，最后读取 `navigator.language`
 *  2. 上述每一步都会确认，读取到的语言标识受支持
 *  3. 有副作用，不可重入；当 url 与 cookie 不一致时，cookie 会用 url 的值进行覆写，以保存设置
 * @returns {string}
 */
function decideCurrentLanguage() {
  let langFromUrl = getSupportLang(getUrlParameter('lang'));
  let langFromCookie = getSupportLang(Cookie.get('lang'));

  if (langFromUrl) {
    if (langFromUrl !== langFromCookie) {
      Cookie.set('lang', langFromUrl);
    }
    return langFromUrl;
  }
  return langFromCookie || getSupportLang(navigator.language);
}

const i18nInstance = new VueI18n({
  locale: decideCurrentLanguage() || process.env.VUE_APP_I18N_LOCALE || 'zh-CN',
  fallbackLocale: process.env.VUE_APP_I18N_FALLBACK_LOCALE || 'zh-CN',
  messages,
});

/**
 * 获取当前应用使用的语言
 * @returns {VueI18n.Locale}
 */
export function getCurrentLanguage() {
  return i18nInstance.locale;
}

/**
 * 将 languageCode 转换成后端使用的 区域标识
 * @param {string} lang
 * @returns {string}
 */
export function languageCode2BELocalCode(lang) {
  return (
    {
      // 'zh-HK': 'zh_HK',
      'zh-CN': 'zh_CN',
      en: 'en_US',
    }[getSupportLang(lang)] || ''
  );
}

export default i18nInstance;
